@@ -14,10 +14,11 @@ public:
     ListViewDelegate(QWidget* parent = 0);
     virtual ~ListViewDelegate();
 
-    enum datarole {headerRole = Qt::UserRole + 100, subHeaderLeftJustifyRole = Qt::UserRole+101, subHeaderRightJustifyRole = Qt::UserRole+102, iconRole = Qt::UserRole+103};
+    enum datarole {headerRole = Qt::UserRole + 100, subHeaderLeftJustifyRole = Qt::UserRole+101, subHeaderRightJustifyRole = Qt::UserRole+102, iconRole = Qt::UserRole+103, highlightItemTextRole = Qt::UserRole+104};
 
     void setHeaderFontPointSize(int size);
     void setSubHeaderFontPointSize(int size);
+    void setItemTextHighlightColor(const QColor& color);
 
 protected:
 
@@ -31,6 +32,7 @@ private:
 
     int m_HeaderFontPointSize;
     int m_SubHeaderFontPointSize;
+    QColor m_TextHighlightColor;
 
     };
 
