@@ -1,26 +1,35 @@
-//QueueView.cpp: Implementation of the QueueView class.
+//QueueView.cpp: Implementation of the QueueView class (view containing the current play queue items, playback controls etc).
 
 #include "QueueView.h"
 
 
-QueueView::QueueView(Spotify::Session* pSession, QWidget* parent)
+QueueView::QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, QWidget* parent)
     : QMainWindow(parent)
 {
-    //construct the QueueView UI
+    //construct the Queue View UI
 
-    setAttribute(Qt::WA_Maemo5StackedWindow);
+    setAttribute(Qt::WA_Maemo5StackedWindow); //window is part of window stack
 
     setWindowTitle(tr("Now Playing"));
 
     m_pSession = pSession; //set the Spotify session instance
 
+    m_playing = false; //set initial playback state
+
+    m_CallerID = QUEUEVIEW_ID;
+
+    m_pOssoWrapper = pOsso; //set OssoWrapper instance pointer
+    if(m_pOssoWrapper)
+    {
+        connect(m_pOssoWrapper,SIGNAL(playbackStateChangeRequest(pb_state_e)),SLOT(OnPlaybackStateChangeRequest(pb_state_e)));
+    }
+
     m_pCurrTrack = NULL;
     m_currQueuePos = 0;
     m_currShufflePos = 0;
-    m_Active = false;
-    m_UpdateBufPos = true;
-    m_Shuffle = false;
-
+    m_currQueueBuffPos = 0;
+    m_shuffle = false;
+    m_shuffleStateChanged = false;
 
     //define slider used for displaying buffering and track progress
     m_pProgressSlider = new CustomSlider(this,SLIDER_WIDTH,SLIDER_HEIGHT);
@@ -46,10 +55,15 @@ QueueView::QueueView(Spotify::Session* pSession, QWidget* parent)
     m_pTrackInfoView->setModel(m_pTrackInfoModel); //connect the model to view
     m_pTrackInfoView->setProperty("FingerScrollable",true); //make sure that the current item is not accidentally de-selected when scrolling
 
+    setupContextMenu(); //setup the listview contextmenu
+
+    setupMenubar(); //setup the view menubar
+
     //set up signal and slot connections
-    connect(m_pSession,SIGNAL(trackPositionUpdated(qint64)),SLOT(OnTrackPosUpdated(qint64)));
+    connect(m_pSession,SIGNAL(trackBufferingAmountUpdated(qint32)),SLOT(OnTrackBufferedAmountUpdated(qint32)));
     connect(m_pSession,SIGNAL(playbackPositionUpdated(qint64)),SLOT(OnPlaybackPositionUpdated(qint64)));
-    connect(m_pSession,SIGNAL(coverArtReady(QImage)),SLOT(OnAlbumCoverReady(QImage)));
+    connect(m_pSession,SIGNAL(coverArtReady(QImage,qint32)),SLOT(OnAlbumCoverReady(QImage,qint32)));
+    connect(m_pSession,SIGNAL(albumBrowserReady(Spotify::AlbumBrowser*,qint32)),SLOT(OnAlbumBrowseReady(Spotify::AlbumBrowser*,qint32)));
     connect(m_pSession,SIGNAL(playing(bool)),SLOT(OnPlayingState(bool)));
     connect(m_pSession,SIGNAL(playbackFinished()),SLOT(OnTrackPlaybackFinished()));
     connect(m_pSession,SIGNAL(bufferNextTrack()),SLOT(OnBufferNextTrack()));
@@ -58,8 +72,9 @@ QueueView::QueueView(Spotify::Session* pSession, QWidget* parent)
     connect(&m_ShuffleBtn,SIGNAL(clicked()),SLOT(OnShuffleToggle()));
     connect(&m_PrevBtn,SIGNAL(clicked()),SLOT(OnPrevTrackBtn()));
     connect(&m_NextBtn,SIGNAL(clicked()),SLOT(OnNextTrackBtn()));
+    connect(menuBar(),SIGNAL(triggered(QAction*)),SLOT(OnMenubarAction(QAction*)));
 
-    m_AlbumCoverContainer.setFixedSize(355,355);
+    m_AlbumCoverContainer.setFixedSize(355,355); //set fixed size of the album coverart
 
     //style the player control buttons
     m_PauseResumeBtn.setCheckable(true); //toggle button (between pause / resume states)
@@ -106,6 +121,7 @@ QueueView::QueueView(Spotify::Session* pSession, QWidget* parent)
     m_PlayControls.addWidget(m_pSpectrum,0,5,1,1,Qt::AlignRight);
     m_PlayControls.setContentsMargins(10,5,10,0);
 
+
     //set up the QueueView UI layout
     m_GridLayout.addWidget(&m_AlbumCoverContainer,0,0,7,1,Qt::AlignTop);
     m_GridLayout.addWidget(m_pTrackInfoView,0,1,7,1);
@@ -119,77 +135,146 @@ QueueView::QueueView(Spotify::Session* pSession, QWidget* parent)
 
 QueueView::~QueueView()
 {
-
 }
 
-void QueueView::setNowPlaying(QList<Spotify::Track*> queue, int currQueuePos, bool playing, qint64 currPlaybackPos, qint64 currBufPos, bool doShuffle, qint32 currShufflePos)
+void QueueView::setupContextMenu()
 {
-        //set the view into play queue (now playing mode), displaying the current play queue
+    //set up the listview context menu
 
-           m_PlayQueue = queue;
+    m_pTrackInfoView->setContextMenuPolicy(Qt::ActionsContextMenu); //actions context menu associated with listview
 
-           if(doShuffle)
-           { //shuffle mode is active, and queue should be shuffle
-               m_currShufflePos = currShufflePos;
-
-               if(m_Shuffle)
-               {
-                   shuffleQueue();
-               }
-           }
-
-           addTracks(m_PlayQueue); //add track data to track list
-
-           if(currQueuePos!=-1)
-           {
-               m_pCurrTrack = m_PlayQueue.at(currQueuePos);
-
-               m_currQueuePos = currQueuePos;
-
-              emit playQueuePosUpdated(m_currQueuePos,m_currShufflePos);
-
-              //m_TrackInfo.setCurrentRow(m_currQueuePos);
-              QModelIndex index = m_pTrackInfoModel->index(m_currQueuePos,0);
-
-              QStandardItem* currTrackItem = m_pTrackInfoModel->itemFromIndex(index);
-              if(currTrackItem)
-                  currTrackItem->setData(true,ListViewDelegate::highlightItemTextRole); //highlight text of current track item
-
-              if(index.isValid())
-                m_pTrackInfoView->setCurrentIndex(index);
+    //set up the listview contextmenu options
+    QAction* pPlayAlbumAct = new QAction(tr("Play Album"),this);
+    connect(pPlayAlbumAct,SIGNAL(triggered()),SLOT(OnPlayAlbumAction()));
+    m_pTrackInfoView->addAction(pPlayAlbumAct);
+    QAction* pViewAlbumAct = new QAction(tr("Browse Album"),this);
+    connect(pViewAlbumAct,SIGNAL(triggered()),SLOT(OnViewAlbumAction()));
+    m_pTrackInfoView->addAction(pViewAlbumAct);
+    QAction* pViewArtistAct = new QAction(tr("Browse Artist"),this);
+    pViewArtistAct->setEnabled(false);
+    //connect(pPlayAlbumAct,SIGNAL(triggered()),SLOT(OnViewArtistAction()));
+    m_pTrackInfoView->addAction(pViewArtistAct);
+}
 
 
-             if(m_pCurrTrack)
-             {
-                 loadTrackData(m_pCurrTrack);
-             }
+void QueueView::setupMenubar()
+{
+    //setup the view menubar
+    menuBar()->addAction(tr("FM Transmitter")); //add FM transmitter button to the menubar
+}
 
-             m_PauseResumeBtn.setEnabled(true);
-             m_PauseResumeBtn.setChecked(playing);
+void QueueView::OnMenubarAction(QAction* action)
+{
+    //Some action in the QueueView menubar has been selected
 
-             //set the state of the progress indicators
-             OnTrackPosUpdated(currBufPos);
-             OnPlaybackPositionUpdated(currPlaybackPos);
-            }
-           else
-           {
-               clearViewSelection();
-           }
+    if(action->text()==tr("FM Transmitter"))
+    {
+        //Request to display the standard FM transmitter configuration dialog
+        //(i.e., requests to execute the control panel FM Transmitter plugin)
+        emit showFMTransmitterDialog();
+    }
+}
+
+void QueueView::show()
+{
+    //overriden show method (handles selection of current playing track etc).
+    QModelIndex index;
+    QStandardItem* currTrackItem = NULL;
+
+    if(m_currQueuePos>=0)
+    {
+        currTrackItem = m_pTrackInfoModel->item(m_currQueuePos);
+        if(currTrackItem)
+        {
+            currTrackItem->setData(true,ListViewDelegate::highlightItemTextRole); //highlight new item text
+        }
+
+        index = m_pTrackInfoModel->index(m_currQueuePos,0);
+        if(index.isValid())
+        {
+            m_pTrackInfoView->setCurrentIndex(index); //set the current row in the queue list
+        }
+    }
+
+    QMainWindow::show(); //call base
+}
+
+void QueueView::OnPlaybackStateChangeRequest(enum pb_state_e requestedState)
+{
+   //request received via libplayback asking for a playback state change
+    if(requestedState==PB_STATE_STOP)
+    { //request to move to stopped state, pause current playback
+        pausePlayback(); //pause playback
+        m_playbackStateChangeReq = true; //indicate that playback state has been changed
+    }
+}
+
+void QueueView::setQueueInitialState(QList<Spotify::Track*> queue)
+{
+    //set the initial state of the play queue view (assign new playqueue, reset members etc.)
+
+    m_PlayQueue = queue;
+
+    addTracks(m_PlayQueue); //add track data to track list
+
+    //set the queue initial state
+
+    m_currQueuePos = 0; //initial playback pos
+    m_currQueueBuffPos = 0; //initial buffering track pos
+    m_shuffleStateChanged = false;
+
+    if(m_shuffle)
+    { //shuffle mode active; shuffle the current play queue
+        shuffleQueue();
+    }
+
+    m_pCurrTrack = m_PlayQueue.at(m_currQueuePos); //set current track to first in queue
+
+    emit playQueuePosUpdated(m_currQueuePos);
+
+    if(m_pCurrTrack)
+    {
+        if(m_pOssoWrapper)
+            m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_PLAY); //request to move into playing state
+        playTrack(m_pCurrTrack); //initiate track playback
+        m_PauseResumeBtn.setEnabled(true);
+        m_PauseResumeBtn.setChecked(true);
+    }
+    else
+    {
+        clearViewSelection();
+    }
+
+    //set the initial state of the progress indicators
+    OnTrackBufferedAmountUpdated(0);
+    OnPlaybackPositionUpdated(0);
 }
 
 void QueueView::clearViewSelection()
 {
     //clear the current view selection (i.e., set to state when no track is active)
 
+    QStandardItem* currTrackItem;
+
     m_AlbumCoverContainer.clear();
 
     m_pTrackInfoView->clearSelection();
 
+    if(m_currQueuePos!=-1)
+    {
+        currTrackItem = m_pTrackInfoModel->item(m_currQueuePos);
+        if(currTrackItem)
+        {
+            currTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight
+        }
+    }
+
     m_pProgressSlider->setPos1(0);
     m_pProgressSlider->setPos2(0);
     m_pProgressSlider->setText("");
 
     m_pCurrTrack = NULL;
+    m_currQueuePos = -1;
 
     m_PauseResumeBtn.setChecked(false);
     m_PauseResumeBtn.setEnabled(false);
@@ -197,65 +282,64 @@ void QueueView::clearViewSelection()
 
 void QueueView::OnPlayingState(bool state)
 {
-    if(m_Active)
-    {
-        if(state)
-        { //playing state
-            m_pSpectrum->Start();
-        }
-        else
-        { //paused state
-           m_pSpectrum->Stop();
-        }
-    }
+   if(state)
+   { //playing state
+      m_playing = true;
+      m_pSpectrum->Start();
+   }
+   else
+   { //paused state
+      m_playing = false;
+      m_pSpectrum->Stop();
+   }
 }
 
 void QueueView::OnBufferNextTrack()
 {
+    //invoked when ready to start buffering the following track (track pre-buffering mode)
+
     Spotify::Error error;
     Spotify::Track* track;
-    int shufflePos;
 
-   //invoked when ready to start buffering the following track (pre-buffering mode)
-    if(m_Active) //current view instance is active
-    {
-        if(m_Shuffle)
-        { //shuffle mode active; get next track position from shuffle queue (if queue is not empty)
-           if(m_currShufflePos < m_ShuffleQueue.size()-1)
-           {
-               shufflePos = m_ShuffleQueue.at(m_currShufflePos+1);
-           }
-           else
-           {
-               shufflePos = m_PlayQueue.size()-1;
-           }
-        }
-
-        if((!m_Shuffle && (m_currQueuePos < m_PlayQueue.size()-1))||(m_Shuffle && (shufflePos < m_PlayQueue.size())))
+    if(m_shuffle)
+    { //shuffle mode active; get next track position from shuffle queue (if queue is not empty)
+        if(m_currShuffleBuffPos < m_ShuffleQueue.size()-1)
         {
-            if(!m_Shuffle)
-                track = m_PlayQueue.at(m_currQueuePos+1);
-            else
-                track = m_PlayQueue.at(shufflePos);
-            if(m_pCurrTrack)
-            {
-                m_pSession->setNewTrackStarted(true); //indicate start of new track audio buffer data (in playback thread FIFO)
-                error = m_pSession->play(track,true); //initiate buffering of track
-                if(error.isError())
-                {
-                    QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
-                }
-                m_UpdateBufPos = false; //do not update the buffering position (current track still playing)
-            }
+            m_currShuffleBuffPos++;
+            qDebug() << "currShuffleBuffPos: " << m_currShuffleBuffPos;
+            m_currQueueBuffPos = m_ShuffleQueue.at(m_currShuffleBuffPos);
+            qDebug() << "currQueueBuffPos: " << m_currQueueBuffPos;
         }
         else
         {
-
-          //end of playqueue reached, set next track pos to end of current track
-            m_pSession->setNewTrackStarted(false);
-            m_pSession->setNextTrackStartPos();
+           m_currQueueBuffPos = m_PlayQueue.size();
         }
     }
+
+    if((!m_shuffle && (m_currQueueBuffPos < m_PlayQueue.size()-1))||(m_shuffle && (m_currQueueBuffPos < m_PlayQueue.size())))
+    {
+        if(!m_shuffle)
+        {
+            m_currQueueBuffPos++;
+        }
+        track = m_PlayQueue.at(m_currQueueBuffPos);
+        if(track)
+        {
+            m_pSession->setNewTrackStarted(true); //indicate start of new track audio buffer data (in playback thread FIFO)
+            error = m_pSession->play(track,true); //initiate buffering of track
+            if(error.isError())
+            {
+                QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
+            }
+        }
+    }
+    else
+    {
+        //end of playqueue reached, set next track pos to end of current track
+        m_currQueueBuffPos = -1;
+        m_pSession->setNewTrackStarted(false);
+        m_pSession->setNextTrackStartPos();
+    }
 }
 
 void QueueView::loadTrackData(Spotify::Track* track)
@@ -269,7 +353,7 @@ void QueueView::loadTrackData(Spotify::Track* track)
     {
         if(track->getAlbum())
         {
-            m_pSession->getAlbumCover(track->getAlbum()); //request to load album coverart
+            m_pSession->getAlbumCover(track->getAlbum(),&m_CallerID); //request to load album coverart
         }
 
         m_currTrackDuration = track->getDuration().hour()*3600+track->getDuration().minute()*60+track->getDuration().second(); //current track duration (in secs)
@@ -300,6 +384,7 @@ void QueueView::addTracks(QList<Spotify::Track*> trackList)
      {
              Spotify::Track* currTrack = trackList.at(trackPos);
              QStandardItem* trackInfo = new QStandardItem();
+             trackInfo->setEditable(false);
              if(currTrack)
              {
                  strSubHeader = "";
@@ -325,6 +410,9 @@ void QueueView::OnTrackActivated(QModelIndex index)
 
         Spotify::Error error;
 
+        if(m_shuffleStateChanged)
+            m_shuffleStateChanged=false;
+
         QStandardItem* prevTrackItem = m_pTrackInfoModel->item(m_currQueuePos); //get previously selected item
         if(prevTrackItem)
             prevTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight from prev. selected item
@@ -333,78 +421,93 @@ void QueueView::OnTrackActivated(QModelIndex index)
         currTrackItem->setData(true,ListViewDelegate::highlightItemTextRole);
 
         m_currQueuePos = index.row(); //get corresponding position in playqueue
+        m_currQueueBuffPos = m_currQueuePos; //set queue buffering position to current queue pos
 
         m_pCurrTrack = m_PlayQueue.at(m_currQueuePos);
 
         if(m_pCurrTrack) //we have got a valid track instance
         {
-            m_pSession->resetPlaybackMembers();
-            loadTrackData(m_pCurrTrack);
-            m_PauseResumeBtn.setChecked(true); //toggle playing state
-            m_pProgressSlider->setPos1(0);
-            m_pProgressSlider->setPos2(0);
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
-            QMaemo5InformationBox::information(this,tr("Buffering"));
-            m_UpdateBufPos = true; //allow update of buffering state
-            error = m_pSession->play(m_pCurrTrack); //request to start playback of track
-            //emit playing();
-            if(error.isError())
-            { //error occured
-                QMaemo5InformationBox::information(this,tr("Error starting track playback: ")+error.toString());
-            }
-            else
-            {
-                m_PauseResumeBtn.setChecked(true);
-                m_PauseResumeBtn.setEnabled(true);
-            }
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-            emit playQueuePosUpdated(m_currQueuePos,m_currShufflePos);
+            playTrack(m_pCurrTrack);
+            emit playQueuePosUpdated(m_currQueuePos);
         }
 }
 
-void QueueView::OnTrackPosUpdated(qint64 trackPos)
+void QueueView::playTrack(Spotify::Track* track)
 {
-    //Track buffering position has been updated
+    //request to play (stream) specified track (and load associated track data)
 
-    if(m_Active && m_UpdateBufPos)
+    Spotify::Error error;
+
+    if(m_playbackStateChangeReq)
+    { //ask for state change
+        m_playbackStateChangeReq = false;
+        m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_PLAY);
+    }
+    m_pSession->resetPlaybackMembers();
+    loadTrackData(track);
+    m_PauseResumeBtn.setChecked(true); //toggle playing state
+    m_pProgressSlider->setPos1(0);
+    m_pProgressSlider->setPos2(0);
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
+    QMaemo5InformationBox::information(this,tr("Buffering"));
+    error = m_pSession->play(track); //request to start playback of track
+    if(error.isError())
+    { //error occured
+        QMaemo5InformationBox::information(this,tr("Error starting track playback: ")+error.toString());
+    }
+    else
+    { //set state of pause / resume button
+        m_PauseResumeBtn.setChecked(true);
+        m_PauseResumeBtn.setEnabled(true);
+    }
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
+}
+
+void QueueView::OnTrackBufferedAmountUpdated(qint32 bufferedAmount)
+{
+    //Amount of buffered of the currently loaded / playing track has been updated
+    int currPos;
+
+    if(m_currQueuePos != m_currQueueBuffPos)
     {
-        int currPos = (trackPos*SLIDER_WIDTH) / (m_currTrackDuration*1000);
+        currPos = SLIDER_WIDTH; //current track has been completely buffered
+    }
+    else
+    {
+        currPos = (bufferedAmount*SLIDER_WIDTH) / (m_currTrackDuration*1000); //current track has partly been buffered
+    }
+    if(currPos!=m_pProgressSlider->getPos1())
         m_pProgressSlider->setPos1(currPos); //set curr. slider position
-    }
-
 }
 
-void QueueView::OnPlaybackPositionUpdated(qint64 trackPos)
+void QueueView::OnPlaybackPositionUpdated(qint64 playbackPos)
 {
-    //Track playback position updated
+    //Current track playback position updated
 
-    if(m_Active)
-    {
+    QString strCurrTrackProgressInfo, strCurrPlaybackPos, strCurrTrackDuration;
 
-        QString strCurrTrackProgressInfo, strCurrPlaybackPos, strCurrTrackDuration;
+    QTextStream outStream(&strCurrTrackProgressInfo);
 
-        QTextStream outStream(&strCurrTrackProgressInfo);
+    QTime currPlaybackPos(playbackPos/3600000,playbackPos/60000,(playbackPos/1000)%60,0);
 
-        QTime currTrackPos(trackPos/3600000,trackPos/60000,(trackPos/1000)%60,0);
-
-        if(m_pCurrTrack)
-        { //set track progress info in format current playback position / total track duration
-                if(currTrackPos.hour()>0)
-                    strCurrPlaybackPos = currTrackPos.toString("h:mm:ss"); //position exceeds 60 mins
-                else
-                    strCurrPlaybackPos = currTrackPos.toString("m:ss");
-                if(m_pCurrTrack->getDuration().hour()>0) //duration longer than 60 mins
-                    strCurrTrackDuration = m_pCurrTrack->getDuration().toString("h:mm:ss");
-                else
-                    strCurrTrackDuration = m_pCurrTrack->getDuration().toString("m:ss");
-                outStream << strCurrPlaybackPos << " / " << strCurrTrackDuration;
-                m_pProgressSlider->setText(strCurrTrackProgressInfo);
-        }
-
-        int currPos = (trackPos*SLIDER_WIDTH) / (m_currTrackDuration*1000);
-
-        m_pProgressSlider->setPos2(currPos); //set curr. slider position
+    if(m_pCurrTrack)
+    { //set track progress info in format current playback position / total track duration
+        if(currPlaybackPos.hour()>0)
+            strCurrPlaybackPos = currPlaybackPos.toString("h:mm:ss"); //position exceeds 60 mins
+        else
+            strCurrPlaybackPos = currPlaybackPos.toString("m:ss");
+        if(m_pCurrTrack->getDuration().hour()>0) //duration longer than 60 mins
+            strCurrTrackDuration = m_pCurrTrack->getDuration().toString("h:mm:ss");
+        else
+            strCurrTrackDuration = m_pCurrTrack->getDuration().toString("m:ss");
+        outStream << strCurrPlaybackPos << " / " << strCurrTrackDuration;
+        m_pProgressSlider->setText(strCurrTrackProgressInfo);
     }
+
+    int currPos = (playbackPos*SLIDER_WIDTH) / (m_currTrackDuration*1000);
+
+    if(currPos!=m_pProgressSlider->getPos2())
+        m_pProgressSlider->setPos2(currPos); //set curr. slider position
 }
 
 void QueueView::OnPauseResumeToggle()
@@ -413,50 +516,25 @@ void QueueView::OnPauseResumeToggle()
 
     if(m_pCurrTrack) //some track has been activated
     {
+        if(m_playbackStateChangeReq)
+        { //ask for state change
+            m_playbackStateChangeReq = false;
+            m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_PLAY);
+        }
         m_pSession->pausePlayback(); //pause / resume playback of current track
     }
 }
 
-void QueueView::setUpdateBufferPos(bool update)
+void QueueView::pausePlayback()
 {
-    //public method for setting the state of buffer progress updating
-    m_UpdateBufPos = update;
-    if(!update)
-    { //set the buffer progress to progress indicator end position (current track has been completely buffered)
-        m_pProgressSlider->setPos1(SLIDER_WIDTH);
-    }
-}
-
-bool QueueView::getUpdateBufferPos()
-{
-    return m_UpdateBufPos;
-}
-
-void QueueView::setActive(bool active, bool shuffleActive)
-{
-    //set whether the current instance is active (i.e., whether signals should be processed),
-    //as well as whether the shuffle function is active.
-    m_Active = active;
-
-    m_Shuffle = shuffleActive;
-    if(m_Shuffle)
+    //request to pause music playback (and set the UI to corresponding state)
+    if(m_playing)
     {
-        shuffleQueue();
-        m_ShuffleBtn.setChecked(true);
-        setWindowTitle(tr("Now Playing") + " - " + tr("Shuffle"));
-    }
-    else
-    {
-        m_ShuffleBtn.setChecked(false);
-        setWindowTitle(tr("Now Playing"));
+        m_pSession->pausePlayback(); //pause playback
+        m_PauseResumeBtn.setChecked(false); //set to paused state
     }
 }
 
-bool QueueView::getShuffleActive()
-{
-    //returns whether the shuffle mode is active
-    return m_Shuffle;
-}
 
 void QueueView::OnNextTrackBtn()
 {
@@ -466,11 +544,14 @@ void QueueView::OnNextTrackBtn()
 
     qint32 oldQueuePos = m_currQueuePos;
 
-    if(m_Shuffle)
+    if(m_shuffleStateChanged)
+        m_shuffleStateChanged=false;
+
+    if(m_shuffle)
     { //shuffle mode active, get position of next track to play from shuffle queue
         if(m_currShufflePos < m_ShuffleQueue.size()-1)
         {
-             m_currShufflePos++;
+            m_currShufflePos++;
             m_currQueuePos = m_ShuffleQueue.at(m_currShufflePos);
         }
         else
@@ -479,10 +560,13 @@ void QueueView::OnNextTrackBtn()
         }
     }
 
-    if((!m_Shuffle && (m_currQueuePos < m_PlayQueue.size()-1))||(m_Shuffle && (m_currQueuePos < m_PlayQueue.size())))
+    if((!m_shuffle && (m_currQueuePos < m_PlayQueue.size()-1))||(m_shuffle && (m_currQueuePos < m_PlayQueue.size())))
     {
-        if(!m_Shuffle)
+        if(!m_shuffle)
+        {
             m_currQueuePos++;
+        }
+        m_currQueueBuffPos = m_currQueuePos;
         QStandardItem* prevTrackItem = m_pTrackInfoModel->item(oldQueuePos); //get previously selected item
         if(prevTrackItem)
             prevTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight from prev. selected item
@@ -494,22 +578,8 @@ void QueueView::OnNextTrackBtn()
             QModelIndex index = m_pTrackInfoModel->index(m_currQueuePos,0);
             if(index.isValid())
               m_pTrackInfoView->setCurrentIndex(index);
-            m_pProgressSlider->setPos1(0);
-            m_pProgressSlider->setPos2(0);
-            m_pSession->resetPlaybackMembers();
-            loadTrackData(m_pCurrTrack);
-            qDebug()<<"Starting track at pos " << m_currQueuePos << " in playqueue";
-            m_PauseResumeBtn.setChecked(true); //toggle playing state
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
-            QMaemo5InformationBox::information(this,tr("Buffering"));
-            m_UpdateBufPos = true; //allow update of buffering state
-            error = m_pSession->play(m_pCurrTrack); //initiate track playback
-            if(error.isError())
-            {
-                QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
-            }
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-            emit playQueuePosUpdated(m_currQueuePos,m_currShufflePos); //emit pos updated
+            playTrack(m_pCurrTrack);
+            emit playQueuePosUpdated(m_currQueuePos); //emit pos updated
         }
     }
 }
@@ -522,7 +592,10 @@ void QueueView::OnPrevTrackBtn()
 
     qint32 oldQueuePos = m_currQueuePos;
 
-    if(m_Shuffle)
+    if(m_shuffleStateChanged)
+        m_shuffleStateChanged=false;
+
+    if(m_shuffle)
     { //shuffle mode active, get position of next track to play from shuffle queue
         if(m_currShufflePos > 0)
         {
@@ -535,10 +608,13 @@ void QueueView::OnPrevTrackBtn()
         }
     }
 
-    if((!m_Shuffle && (m_currQueuePos > 0))||(m_Shuffle && (m_currQueuePos>=0)))
+    if((!m_shuffle && (m_currQueuePos > 0))||(m_shuffle && (m_currQueuePos>=0)))
     {
-        if(!m_Shuffle)
+        if(!m_shuffle)
+        {
             m_currQueuePos--;
+        }
+        m_currQueueBuffPos = m_currQueuePos;
         QStandardItem* prevTrackItem = m_pTrackInfoModel->item(oldQueuePos); //get previously selected item
         if(prevTrackItem)
             prevTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight from prev. selected item
@@ -550,117 +626,124 @@ void QueueView::OnPrevTrackBtn()
             QModelIndex index = m_pTrackInfoModel->index(m_currQueuePos,0);
             if(index.isValid())
               m_pTrackInfoView->setCurrentIndex(index);
-            m_pProgressSlider->setPos1(0);
-            m_pProgressSlider->setPos2(0);
-            m_pSession->resetPlaybackMembers();
-            loadTrackData(m_pCurrTrack);
-            qDebug()<<"Starting track at pos " << m_currQueuePos << " in playqueue";
-            m_PauseResumeBtn.setChecked(true); //toggle playing state
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
-            QMaemo5InformationBox::information(this,tr("Buffering"));
-            m_UpdateBufPos = true; //allow update of buffering state
-            error = m_pSession->play(m_pCurrTrack); //initiate track playback
-            if(error.isError())
-            {
-                QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
-            }
-             setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-            emit playQueuePosUpdated(m_currQueuePos,m_currShufflePos); //emit pos updated
+            playTrack(m_pCurrTrack);
+            emit playQueuePosUpdated(m_currQueuePos); //emit pos updated
         }
     }
 }
 
 void QueueView::OnTrackPlaybackFinished()
 {
-    if(m_Active)
-    {
-        //Playback of current track has finished
-        //move to next track in play queue, if possible
+   //Playback of current track has finished
+   //move to next track in play queue, if possible
 
-        qDebug() << "Playback finished..";
+    qDebug() << "Playback finished..";
 
-        Spotify::Error error;
+    Spotify::Error error;
 
-        qint32 oldQueuePos = m_currQueuePos;
+    qint32 oldQueuePos = m_currQueuePos; //save current (old) queue position
+    qint32 oldShufflePos = m_currShufflePos;
 
-        if(m_Shuffle)
-        { //shuffle mode active, get position of next track to play from shuffle queue
-            if(m_currShufflePos<m_ShuffleQueue.size()-1)
-            {
-                m_currShufflePos++;
-                m_currQueuePos = m_ShuffleQueue.at(m_currShufflePos);
-            }
-            else
-            { //no tracks left in shuffle queue
-                m_currQueuePos = m_PlayQueue.size();
-            }
-        }
-
-        if((!m_Shuffle && (m_currQueuePos < m_PlayQueue.size()-1))||(m_Shuffle && (m_currQueuePos<m_PlayQueue.size())))
+    if(m_shuffle)
+    { //shuffle mode active, get position of next track to play from shuffle queue
+        if(m_currShufflePos<m_ShuffleQueue.size()-1)
         {
-            if(!m_Shuffle) //normal playback mode
-                m_currQueuePos++;
-            QStandardItem* prevTrackItem = m_pTrackInfoModel->item(oldQueuePos); //get previously selected item
-            if(prevTrackItem)
-                prevTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight from prev. selected item
-            QStandardItem* currTrackItem = m_pTrackInfoModel->item(m_currQueuePos);
-            currTrackItem->setData(true,ListViewDelegate::highlightItemTextRole); //highlight new item text
-            m_pCurrTrack = m_PlayQueue.at(m_currQueuePos);
-            if(m_pCurrTrack)
-            {
-                m_pProgressSlider->setPos1(0);
-                m_pProgressSlider->setPos2(0);
-                QModelIndex index = m_pTrackInfoModel->index(m_currQueuePos,0);
-                if(index.isValid())
-                  m_pTrackInfoView->setCurrentIndex(index);
-                 loadTrackData(m_pCurrTrack);
-                if(!m_pSession->getPreBufferingActive()) //pre-buffering is not active, buffering / playback should be initiated
-                {
-                    m_pSession->resetPlaybackMembers();
-                    error = m_pSession->play(m_pCurrTrack); //initiate track playback
-                    if(error.isError())
-                    {
-                        QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
-                    }
-                }
-                else
-                { //track pre-buffering mode active; reset playback position and playing state
-                    m_pSession->setPlaybackPos(0); //reset current playback position
-                    m_UpdateBufPos = true; //allow update of buffering state
-                    OnTrackPosUpdated(m_pSession->getBufferingPos()); //update the current buffering state
-                }
-                emit playQueuePosUpdated(m_currQueuePos,m_currShufflePos); //emit pos updated
-            }
+            m_currShufflePos++;
+            m_currQueuePos = m_ShuffleQueue.at(m_currShufflePos);
         }
         else
-        {
-            //end of playback queue reached
-            clearViewSelection();
-            emit playQueuePosUpdated(-1,m_currShufflePos); //emit pos updated (indicates that playqueue has reached the end)
+        { //no tracks left in shuffle queue
+            m_currQueuePos = m_PlayQueue.size();
         }
     }
+
+    if((!m_shuffle && (m_currQueuePos < m_PlayQueue.size()-1))||(m_shuffle && (m_currQueuePos < m_PlayQueue.size())))
+    {
+        if(!m_shuffle) //normal playback mode
+            m_currQueuePos++;
+        QStandardItem* prevTrackItem = m_pTrackInfoModel->item(oldQueuePos); //get previously selected item
+        if(prevTrackItem)
+            prevTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight from prev. selected item
+        QStandardItem* currTrackItem = m_pTrackInfoModel->item(m_currQueuePos);
+        currTrackItem->setData(true,ListViewDelegate::highlightItemTextRole); //highlight new item text
+        m_pCurrTrack = m_PlayQueue.at(m_currQueuePos);
+        if(m_pCurrTrack)
+        {
+            m_pProgressSlider->setPos1(0);
+            m_pProgressSlider->setPos2(0);
+            QModelIndex index = m_pTrackInfoModel->index(m_currQueuePos,0);
+            if(index.isValid())
+                m_pTrackInfoView->setCurrentIndex(index);
+            loadTrackData(m_pCurrTrack);
+            if(!m_pSession->getPreBufferingActive()) //pre-buffering is not active, buffering / playback should be initiated
+            {
+                m_pSession->resetPlaybackMembers();
+                error = m_pSession->play(m_pCurrTrack); //initiate track playback
+                if(error.isError())
+                {
+                    QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
+                }
+            }
+            else
+            { //track pre-buffering mode active; reset playback position and playing state
+                m_pSession->setPlaybackPos(0); //reset current playback position
+                if(m_currQueueBuffPos==-1)
+                    m_pProgressSlider->setPos1(SLIDER_WIDTH); //set buffering progress to 100% (final track buffered completely)
+                qDebug() << m_currQueueBuffPos << "," << oldQueuePos;
+                if(m_currQueueBuffPos != oldQueuePos)
+                {
+                    if(m_shuffleStateChanged)
+                    {//shuffle state has been changed (enabled / disabled), and possible pre-buffered track data should be discarded
+                        m_shuffleStateChanged = false;
+                        m_currQueueBuffPos = m_currQueuePos; //buffering the currently playing track
+                        m_currShuffleBuffPos = m_currShufflePos;
+                        m_pSession->resetPlaybackMembers();
+                        error = m_pSession->play(m_pCurrTrack); //initiate track playback
+                        if(error.isError())
+                        {
+                            QMaemo5InformationBox::information(this,tr("Error starting track playback: ") + error.toString());
+                        }
+                    }
+                }
+            }
+            emit playQueuePosUpdated(m_currQueuePos); //emit pos updated
+        }
+    }
+    else
+    {
+        //end of playback queue reached
+        clearViewSelection();
+        emit playQueuePosUpdated(-1); //emit pos updated (indicates that playqueue has reached the end)
+    }
 }
 
-void QueueView::OnAlbumCoverReady(QImage image)
+void QueueView::OnAlbumCoverReady(QImage image, qint32 callerID)
 {
     //Album coverart has been loaded (for currently selected track),
     //assign it to the album coverart container.
-    m_AlbumCoverContainer.setPixmap(QPixmap::fromImage(image).scaled(355,355));
+
+    if(callerID==m_CallerID)
+    {
+        m_AlbumCoverContainer.setPixmap(QPixmap::fromImage(image).scaled(355,355));
+    }
 }
 
 void QueueView::OnShuffleToggle()
 {
-    //toggle shuffle state (on / off)
+    //toggle the shuffle state (on / off)
 
-    if(m_Shuffle)
+    if(m_currQueuePos!=m_currQueueBuffPos)
+        m_shuffleStateChanged = true; //indicate that current shuffle state has changed (enabled / disabled)
+
+    if(m_shuffle)
     {
-        m_Shuffle = false;
+        m_shuffle = false;
         setWindowTitle(tr("Now Playing"));
     }
     else
     {
         shuffleQueue(); //shuffle the current playqueue
-        m_Shuffle = true;
+        m_shuffle = true;
         setWindowTitle(tr("Now Playing") + " - " + tr("Shuffle"));
     }
 }
@@ -689,6 +772,7 @@ void QueueView::shuffleQueue()
     }
 
     m_currShufflePos = -1;
+    m_currShuffleBuffPos = -1;
 }
 
 void QueueView::keyPressEvent(QKeyEvent *evt)
@@ -705,3 +789,70 @@ void QueueView::keyPressEvent(QKeyEvent *evt)
         QMainWindow::keyPressEvent(evt);
     }
 }
+
+
+void QueueView::OnPlayAlbumAction()
+{
+    //request to play the album of currently selected track
+
+    Spotify::Track* track = m_PlayQueue.at(m_pTrackInfoView->currentIndex().row());
+
+    if(track)
+    {
+        if(track->getAlbum())
+        {
+            if(m_pSession->browse(track->getAlbum(),&m_CallerID)) //request to load album tracks (browse album)
+            {
+                m_pTrackInfoView->setDisabled(true);
+                QMaemo5InformationBox::information(this,tr("Loading album tracks"));
+                setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true);
+            }
+        }
+    }
+}
+
+void QueueView::OnAlbumBrowseReady(Spotify::AlbumBrowser* browser, qint32 callerID)
+{
+    //notification that album browse request completed
+
+    Spotify::Error error;
+
+    if(callerID==m_CallerID)
+    {
+        if(browser)
+        {
+            error = sp_albumbrowse_error(browser->getAlbumBrowser());
+            if(error.isError())
+            {
+                //error occured (album browse request failed)
+                QMaemo5InformationBox::information(this,tr("Fetching of album tracks failed: ") + error.toString());
+            }
+            else
+            {
+                if(!browser->load())
+                {
+                    QMaemo5InformationBox::information(this,tr("Failed to load tracks!"));
+                }
+                browser->sortByDiscIndex(); //sort tracks in album order
+                setQueueInitialState(browser->getTracks(false)); //only add available (playable) album tracks to the playqueue
+            }
+        }
+    }
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false);
+    m_pTrackInfoView->setDisabled(false);
+}
+
+void QueueView::OnViewAlbumAction()
+{
+    //Request to show Album View for currently selected album.
+
+    Spotify::Track* track = m_PlayQueue.at(m_pTrackInfoView->currentIndex().row());
+
+    if(track)
+    {
+        if(track->getAlbum())
+        {
+            emit showAlbumView(track->getAlbum());
+        }
+    }
+}
