@@ -13,6 +13,7 @@
 #include <QtGui/QListView>
 #include <QStandardItemModel>
 #include <QItemSelectionModel>
+#include <QMenuBar>
 #include <QModelIndex>
 #include <QStandardItem>
 #include <QDebug>
@@ -21,34 +22,33 @@
 #include <QKeyEvent>
 #include <QtGui/QPixmap>
 #include <QMaemo5InformationBox>
+#include <QAction>
 #include <qspotify_src/spotifysession.h>
 #include "CustomSlider.h"
 #include "SpectrumAnalyzer.h"
 #include "ListViewDelegate.h"
+#include "OssoWrapper.h"
 
 //defines
 
 #define SLIDER_WIDTH 355 //width of the custom progress slider widget
 #define SLIDER_HEIGHT 40 //height of the custom progress slider widget
 
+#define QUEUEVIEW_ID 1
+
 class QueueView : public QMainWindow
 {
     Q_OBJECT
 
 public:
 
-    QueueView(Spotify::Session* pSession, QWidget* parent = 0); //constructor
+    QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, QWidget* parent = 0); //constructor
     ~QueueView();
 
-    void setNowPlaying(QList<Spotify::Track*> queue, int currQueuePos, bool playing, qint64 currPlaybackPos = 0, qint64 currBufPos = 0, bool doShuffle = false, qint32 currShufflePos=-1);
+    void setQueueInitialState(QList<Spotify::Track*> queue); //public method for setting the initial playqueue view state
 
-    void setActive(bool active, bool shuffleActive=false); //set whether the current instance is active (i.e., whether signals should be processed)
+    virtual void show(); //overriden show method
 
-    void setUpdateBufferPos(bool update);
-
-    bool getShuffleActive();
-
-    bool getUpdateBufferPos();
 
 protected:
 
@@ -80,7 +80,7 @@ private:
 
     SpectrumAnalyzer* m_pSpectrum; //spectrum (active during audio playback)
 
-    QListView* m_pTrackInfoView;
+    QListView* m_pTrackInfoView; //track info listview
 
     QStandardItemModel *m_pTrackInfoModel;
 
@@ -88,23 +88,33 @@ private:
 
     Spotify::Session* m_pSession; //Spotify session instance
 
+    OssoWrapper* m_pOssoWrapper; //OssoWrapper instance pointer
+
     Spotify::Track* m_pCurrTrack; //currently playing track in queue
 
-    bool m_Shuffle; //TRUE in case shuffle function enabled
+    bool m_shuffle; //TRUE in case shuffle function enabled
 
-    qint32 m_currShufflePos; //current position in the shuffle queue
+    bool m_shuffleStateChanged; //TRUE in case shuffle state has been changed
 
-    qint32 m_currQueuePos; //current position in play queue
+    qint32 m_currShufflePos; //current position in the shuffle queue (playing)
+
+    qint32 m_currShuffleBuffPos; //current position in shuffle queue being buffered (used in track pre-buffering mode)
+
+    qint32 m_currQueuePos; //current position in play queue (track currently being played)
+
+    qint32 m_currQueueBuffPos; //current position in playqueue being buffered (used in track pre-buffering mode)
 
     qint32 m_currTrackDuration; //current track duration (in secs)
 
+    bool m_playing; //TRUE in case currently in playing state
+
+    qint32 m_CallerID; //used as identifier to Spotify session calls that require caller ID
+
     QList<Spotify::Track*> m_PlayQueue; //current play queue
 
     QList<qint32> m_ShuffleQueue; //list holds random track positions (used by shuffle function)
 
-    bool m_Active; //TRUE in case current queueview instance is active (i.e., handles signals)
-
-    bool m_UpdateBufPos; //TRUE in case buffering position should be updated
+    bool m_playbackStateChangeReq; //TRUE when i.e., the playback state needs to be chantged (to playing)
 
     void addTracks(QList<Spotify::Track*> trackList); //add tracks from specified tracklist to main listview
 
@@ -114,25 +124,39 @@ private:
 
     void shuffleQueue(); //shuffle the current playqueue (the m_ShuffleQueue contains the shuffled queue positions)
 
+    void playTrack(Spotify::Track* track); //request to play (stream) specified track
+
+    void pausePlayback(); //request to pause music playback (and set the UI to corresponding state)
+
+    void setupContextMenu(); //set up the listview context menu
+    void setupMenubar(); //set up the view menubar
+
+
 
 private slots:
 
     void OnTrackActivated(QModelIndex index);
     void OnPauseResumeToggle();
     void OnShuffleToggle();
-    void OnTrackPosUpdated(qint64 trackPos);
-    void OnPlaybackPositionUpdated(qint64 trackPos);
-    void OnAlbumCoverReady(QImage image);
+    void OnTrackBufferedAmountUpdated(qint32 bufferedAmount);
+    void OnPlaybackPositionUpdated(qint64 playbackPos);
+    void OnAlbumCoverReady(QImage image, qint32 callerID);
     void OnTrackPlaybackFinished();
     void OnPlayingState(bool state);
     void OnNextTrackBtn();
     void OnPrevTrackBtn();
     void OnBufferNextTrack();
+    void OnMenubarAction(QAction* action);
+    void OnPlaybackStateChangeRequest(enum pb_state_e requestedState);
+    void OnViewAlbumAction();
+    void OnPlayAlbumAction();
+    void OnAlbumBrowseReady(Spotify::AlbumBrowser* browser, qint32 callerID);
 
 signals:
 
-    void playQueuePosUpdated(qint32, qint32); //signal emitted when position in playqueue has been updated
-
+    void playQueuePosUpdated(qint32); //signal emitted when position in playqueue has been updated
+    void showFMTransmitterDialog(); //signal emitted upon request to show the FM Transmitter control panel plugin
+    void showAlbumView(Spotify::Album*); //request to show the album view for the specified album
 };
 
 #endif // QUEUEVIEW_H
