@@ -9,6 +9,7 @@
 #include <QWidgetAction>
 #include <QMenuBar>
 #include <QStringList>
+#include <QPoint>
 #include <QListView>
 #include <QStandardItemModel>
 #include <QModelIndex>
@@ -16,14 +17,12 @@
 #include <QPixmap>
 #include <QIcon>
 #include <QDebug>
-#include <QCloseEvent>
 #include <QKeyEvent>
 #include <QMaemo5InformationBox>
-#include <qspotify_src/spotifysession.h>
 #include "ListViewDelegate.h"
 #include "PickSelectorAction.h"
 #include "CListView.h"
-#include "QueueView.h"
+#include <qspotify_src/spotifysession.h>
 
 
 //possible sorting alternatives
@@ -37,6 +36,8 @@
 #define ALBUM_ASC    6
 #define ALBUM_DESC   7
 
+#define  TRACKVIEW_ID 3
+
 class TrackView : public QMainWindow
 {
     Q_OBJECT
@@ -48,14 +49,8 @@ public:
     void setPlaylist(Spotify::Playlist* pPlaylist); //set the playlist for which tracks should be displayed
     void setSearchResults(Spotify::Search* pSearch); //set the search results to be displayed
 
-    void setShuffleActive(bool active); //set whether shuffle mode is active
-
-    bool getQueueViewUpdateBufPos(); //returns the update buffer position status for the TrackView QueueView instance
-
 protected:
 
-    void closeEvent(QCloseEvent* evt);
-
     void keyPressEvent(QKeyEvent *evt);
 
 private:
@@ -72,8 +67,6 @@ private:
 
     QList<Spotify::Track*> m_TrackList; //list of tracks returned by e.g., search and contained in playlist
 
-    QueueView *m_pQueueView;
-
     QStandardItemModel *m_pTrackListModel;
 
     ListViewDelegate *m_pTrackListDelegate;
@@ -86,8 +79,9 @@ private:
 
     PickSelectorAction* m_pSortByAct;
 
-    bool m_Shuffle; //TRUE if Shuffle mode currently active
+    qint32 m_CallerID; //passed as identifer to Spotify session methods requiring caller ID
 
+    Spotify::Track* getTrackFromModelIndex(QModelIndex index); //get track instance pointer from model index
 
     void addTrackListModelItem(Spotify::Track* track, int pos); //append track to the track list item model
 
@@ -97,19 +91,24 @@ private:
 
     void setPlayQueue(); //set playqueue (i.e., add all tracks from selected track to last track in list to playqueue)
 
+    void setupContextMenu(); //set up the listview context menu
+    void setupMenubar(); //set up the view menubar
+
 
 private slots:
 
     void OnTrackActivated(QModelIndex trackIndex);
-    void OnPlayQueuePosUpdated(qint32 pos,qint32 shufflePos);
     void OnSortingSelectionChanged(int currSelection);
+    void OnMenubarAction(QAction* action);
+    void OnViewAlbumAction();
+    void OnPlayAlbumAction();
+    void OnAlbumBrowseReady(Spotify::AlbumBrowser* browser, qint32 callerID);
 
 signals:
 
-    void playQueueSet(QList<Spotify::Track*>); //signal emitted upon playqueue set
-    void playQueuePosUpdated(qint32,qint32); //signal emitted when position in playqueue has been updated
-    void activeQueueViewChanged(bool,bool); //signal emitted when active queueview changed (first param is TRUE in case TrackView instance is active)
-                                            //Second param indicates whether shuffle mode is currently active
+    void playQueueSet(const QList<Spotify::Track*>&); //signal emitted upon new playqueue set
+    void showFMTransmitterDialog(); //signal emitted upon request to show the FM Transmitter control panel dialog (plugin)
+    void showAlbumView(Spotify::Album*); //request to show the album view for the specified album
 };
 
 #endif // TRACKVIEW_H
