@@ -6,6 +6,7 @@ MainWindow::MainWindow(QWidget *parent)
     setAttribute(Qt::WA_Maemo5StackedWindow);
 
     QCoreApplication::setApplicationName("QSpot"); //Setup app name
+    QCoreApplication::setApplicationVersion(APP_VERSION);
 
     setWindowTitle(tr("QSpot - Playlists"));
 
@@ -14,28 +15,43 @@ MainWindow::MainWindow(QWidget *parent)
 
     m_pSession = new Spotify::Session(this); //create new Spotify session instance
 
+    m_pOssoWrapper = new OssoWrapper("QSpot", APP_VERSION, this); //create wrapper instance for libosso
+
+    if(m_pOssoWrapper)
+    { //register / declare new playback object with OSSO libplayback (whitelisting support / support for playing in silent profile)
+        m_pOssoWrapper->initOssoPlayback("QSpot");
+        m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_STOP); //request to enter initial stopped state
+    }
+
     m_pTimeoutTimer  = new QTimer(this); //timeout timer instance
     connect(m_pTimeoutTimer,SIGNAL(timeout()),SLOT(OnPlaylistLoadingTimeout()));
 
-    m_pAppSettings = new QSettings("qspot","qspot", this);
+    m_pAppSettings = new QSettings("qspot","qspot", this); //allocate application settings instance
 
-    m_pLoginDlg = new LoginDialog(m_pSession,m_pAppSettings,this); //allocate login dialog instance
+    m_pLoginDlg = new LoginDialog(m_pSession,m_pAppSettings, this); //allocate login dialog instance
 
-    m_pSettingsDlg = new SettingsDialog(m_pAppSettings,this); //allocate settings dialog instance
+    m_pSettingsDlg = new SettingsDialog(m_pAppSettings, this); //allocate settings dialog instance
 
     //set up the the main window menu
     menuBar()->addAction(tr("Settings"));
+    menuBar()->addAction(tr("FM Transmitter"));
     menuBar()->addAction(tr("About"));
 
     //setup the app About dialog box
+    initAboutDialog();
 
-    QPixmap appIconMap;
-    appIconMap.load("/usr/share/icons/hicolor/64x64/apps/qspot.png"); //load application icon from file
+    m_pTrackView = new TrackView(m_pSession,this);
+    connect(m_pTrackView,SIGNAL(playQueueSet(const QList<Spotify::Track*>&)),SLOT(OnSetPlayQueue(const QList<Spotify::Track*>&)));
+    connect(m_pTrackView,SIGNAL(showFMTransmitterDialog()),SLOT(OnDisplayFMTransmitterDialog()));
+    connect(m_pTrackView,SIGNAL(showAlbumView(Spotify::Album*)),SLOT(OnShowAlbumView(Spotify::Album*)));
 
-    m_AboutDlg.setIconPixmap(appIconMap);
-    m_AboutDlg.setWindowTitle(tr("About QSpot")),
-    m_AboutDlg.setText(tr("QSpot version 0.0.5-4\n\nAn open-source Qt-based Spotify client for the N900/Maemo5 (based on libopenspotify).\n\n(C) 2010 Andreas Enbacka and The QSpot Team.\n\n"
-                          "Special thanks to Markus (markusm@tmo) for assistance with the graphical design and layout."));
+    m_pAlbumView = new AlbumView(m_pSession,m_pTrackView);
+    connect(m_pAlbumView,SIGNAL(playQueueSet(const QList<Spotify::Track*>&)),SLOT(OnSetPlayQueue(const QList<Spotify::Track*>&)));
+
+    m_pQueueView = new QueueView(m_pSession,m_pOssoWrapper,this); //initial parent set to current instance
+    connect(m_pQueueView,SIGNAL(playQueuePosUpdated(qint32)),SLOT(OnPlayQueuePosUpdated(qint32)));
+    connect(m_pQueueView,SIGNAL(showFMTransmitterDialog()),SLOT(OnDisplayFMTransmitterDialog()));
+    connect(m_pQueueView,SIGNAL(showAlbumView(Spotify::Album*)),SLOT(OnShowAlbumView(Spotify::Album*)));
 
     //init the playlist listview instance
     m_pPlaylistView = new QListView(this);
@@ -54,7 +70,6 @@ MainWindow::MainWindow(QWidget *parent)
     connect(m_pSession,SIGNAL(playlistContainersReady(QList<Spotify::Playlist*>)),this,SLOT(OnPlaylistContainersReady(QList<Spotify::Playlist*>)));
     connect(m_pSession,SIGNAL(playing(bool)),SLOT(OnPlayingState(bool)));
     connect(m_pSession,SIGNAL(searchComplete(Spotify::Search*)),SLOT(OnSearchComplete(Spotify::Search*)));
-    connect(m_pSession,SIGNAL(trackPositionUpdated(qint64)),SLOT(OnTrackPosUpdated(qint64)));
     connect(m_pSession,SIGNAL(playbackPositionUpdated(qint64)),SLOT(OnPlaybackPositionUpdated(qint64)));
     connect(m_pSession,SIGNAL(playTokenLost()),SLOT(OnPlaybackTokenLost()),Qt::QueuedConnection);
     connect(m_pPlaylistView,SIGNAL(activated(QModelIndex)),SLOT(OnPlaylistActivated(QModelIndex)));
@@ -106,24 +121,37 @@ MainWindow::MainWindow(QWidget *parent)
 
     setCentralWidget(&m_MainLayout);
 
-    m_pTrackView = new TrackView(m_pSession,this);
-
-    connect(m_pTrackView,SIGNAL(playQueueSet(QList<Spotify::Track*>)),SLOT(OnSetPlayQueue(QList<Spotify::Track*>)));
-    connect(m_pTrackView,SIGNAL(playQueuePosUpdated(qint32,qint32)),SLOT(OnPlayQueuePosUpdated(qint32,qint32)));
-    connect(m_pTrackView,SIGNAL(activeQueueViewChanged(bool,bool)),SLOT(OnActiveQueueViewChanged(bool,bool)));
-
-    m_pQueueView = new QueueView(m_pSession,this);
-    connect(m_pQueueView,SIGNAL(playQueuePosUpdated(qint32,qint32)),SLOT(OnPlayQueuePosUpdated(qint32,qint32)));
-
     m_AudioInit = false;
 
     emit showLogin(); //emit signal requesting to show login dialog
 }
 
+void MainWindow::initAboutDialog()
+{
+    //init the application about dialog
+    QPixmap appIconMap;
+    appIconMap.load("/usr/share/icons/hicolor/64x64/apps/qspot.png"); //load application icon from file
+
+    m_AboutDlg.setIconPixmap(appIconMap);
+    m_AboutDlg.setWindowTitle(tr("About QSpot"));
+    m_AboutDlg.setText(tr("QSpot version ") + APP_VERSION + "\n\n" + tr("An open-source Qt-based Spotify client for the N900/Maemo5 (based on libopenspotify).\n\n(C) 2010 Andreas Enbacka and The QSpot Team.\n\n"
+                          "Special thanks to Markus (markusm@tmo) for assistance with the graphical design and layout."));
+}
+
 MainWindow::~MainWindow()
 {
 }
 
+void MainWindow::OnDisplayFMTransmitterDialog()
+{
+    //request to display the standard FMTransmitter configuration dialog (control panel plugin);
+
+    if(m_pOssoWrapper)
+    {
+        m_pOssoWrapper->executePlugin("libcpfmtx.so");
+    }
+}
+
 void MainWindow::OnSettingsSaved()
 {
     //called when app settings have been saved (modified)
@@ -141,11 +169,12 @@ void MainWindow::OnSettingsSaved()
 
 void MainWindow::OnShowLogin()
 {
-  m_pLoginDlg->showMaximized(); //show the login dialog
+    m_pLoginDlg->showMaximized(); //show the login dialog
 }
 
 void MainWindow::OnPlayingState(bool state)
 {
+    //playback state changed notification from session instance
     if(state)
     { //playing state
         m_pSpectrum->Start();
@@ -158,39 +187,6 @@ void MainWindow::OnPlayingState(bool state)
     }
 }
 
-void MainWindow::OnTrackPosUpdated(qint64 pos)
-{
-    //set current buffering position
-    m_currBuffPos = pos;
-
-}
-
-void MainWindow::OnActiveQueueViewChanged(bool active, bool shuffleActive)
-{
-    //notification that active QueueView state has changed.
-    //If active is false, MainWindow owned QueueView instance should be active,
-    //else the TrackView owned is active.
-    //shuffleActive parameter indicates whether shuffling mode should be active
-
-    if(active)
-    {
-        m_pQueueView->setActive(false);
-    }
-    else
-    {
-        if(m_PlayQueue.size()>0)
-        {
-            m_pQueueView->setNowPlaying(m_PlayQueue,m_currQueuePos,m_playing,m_currPlaybackPos,m_currBuffPos,false,-1); //set queue (now playing) instance info
-            m_pQueueView->setUpdateBufferPos(m_pTrackView->getQueueViewUpdateBufPos());
-            if(shuffleActive)
-                m_pQueueView->setActive(true,shuffleActive);
-            else
-                m_pQueueView->setActive(true,m_pQueueView->getShuffleActive());
-
-        }
-    }
-}
-
 void MainWindow::OnPlaybackPositionUpdated(qint64 pos)
 {
         //set current playback position, and update the track progress part of the now playing infobar
@@ -242,24 +238,45 @@ void MainWindow::OnNowPlaying()
     {
         if(m_pQueueView)
         {
-            m_pQueueView->setNowPlaying(m_PlayQueue,m_currQueuePos,m_playing,m_currPlaybackPos,m_currBuffPos,false,m_currShufflePos); //set queue (now playing) instance info
+            if(m_pQueueView->parent()!=this)
+                m_pQueueView->setParent(this,m_pQueueView->windowFlags()); //re-parent QueueView instance (parent set to current instance)
             m_pQueueView->show();
         }
     }
 }
 
-void MainWindow::OnSetPlayQueue(QList<Spotify::Track*> queue)
+void MainWindow::OnSetPlayQueue(const QList<Spotify::Track*>& queue)
 {
-    //set the current (active) playqueue
+    //set the current (active) playqueue to be
     m_PlayQueue = queue;
+    if(sender()!=m_pAlbumView)
+    {
+        if(m_pQueueView->parent()!=sender())
+            m_pQueueView->setParent((QWidget*)sender(),m_pQueueView->windowFlags()); //re-parent the QueueView instance (to sender instance)
+    }
+    else
+    {
+        if(m_pQueueView->parent()!=m_pTrackView)
+            m_pQueueView->setParent(m_pTrackView,m_pQueueView->windowFlags()); //re-parent the QueueView instance (to TrackView instance)
+
+    }
+    m_pQueueView->setQueueInitialState(m_PlayQueue);
+    m_pQueueView->show();
 }
 
-void MainWindow::OnPlayQueuePosUpdated(qint32 pos, qint32 shufflePos)
+void MainWindow::OnShowAlbumView(Spotify::Album* album)
+{
+    //show the Album View for the specified album
+    //m_pAlbumView->setParent((QWidget*)sender(),m_pAlbumView->windowFlags()); //re-parent the Album View instance (to sender instance)
+    m_pAlbumView->setAlbum(album);
+    m_pAlbumView->showMaximized();
+}
+
+void MainWindow::OnPlayQueuePosUpdated(qint32 pos)
 {
      if(pos>=0)
      {
          m_currQueuePos = pos;
-         m_currShufflePos = shufflePos;
 
          Spotify::Track* currTrack = m_PlayQueue.at(m_currQueuePos);
 
@@ -276,8 +293,6 @@ void MainWindow::OnPlayQueuePosUpdated(qint32 pos, qint32 shufflePos)
      }
      else //empty now playing infobar (when playqueue has reached end)
      {
-        m_currQueuePos = -1;
-        m_currShufflePos = -1;
         m_NowPlayingInfo.setText("----");
         m_NowPlayingPositionInfo.setText("");
      }
@@ -326,6 +341,8 @@ void MainWindow::OnPlaylistLoadingTimeout()
 {
     //Timeout has occured during playlist loading, request to interrupt the loading process
 
+    qDebug() << "Playlist loading timeout!";
+
     m_pTimeoutTimer->stop();
 
     if(m_pSession->playlistLoadingInProgress())
@@ -337,6 +354,7 @@ void MainWindow::OnPlaylistLoadingTimeout()
         if(!m_AudioInit)
         {
             m_pSession->initAudio(); //init audio interface
+            m_pSession->setMinBufferSize(m_pAppSettings->value("QSpot/minBufferLen",MIN_BUFFER_LEN_DEFAULT).toInt()); //update min. buffer length (default to 30 secs)
             m_AudioInit = true;
         }
     }
@@ -362,6 +380,7 @@ void MainWindow::OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists)
    {
        Spotify::Playlist* pl = playlists.at(pos);
        QStandardItem *item = new QStandardItem();
+       item->setEditable(false);
        item->setData(listTypeIcon,ListViewDelegate::iconRole);
        item->setData(pl->getName(),ListViewDelegate::headerRole);
        item->setData(QString::number(pl->getNumTracks()) + " " + tr("tracks"),ListViewDelegate::subHeaderLeftJustifyRole);
@@ -375,10 +394,12 @@ void MainWindow::OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists)
 
    if(!m_AudioInit)
    {
-       m_pSession->initAudio(); //init audio interface
+      m_pSession->initAudio(); //init audio interface
+      m_pSession->setMinBufferSize(m_pAppSettings->value("QSpot/minBufferLen",MIN_BUFFER_LEN_DEFAULT).toInt()); //update min. buffer length (default to 30 secs)
        m_AudioInit = true;
    }
 
+
    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false);
 }
 
@@ -393,7 +414,6 @@ void MainWindow::OnPlaylistActivated(QModelIndex index)
         if(m_pTrackView)
         {
             m_pTrackView->setPlaylist(pPlaylist); //set active playlist for track view
-            m_pTrackView->setShuffleActive(m_pQueueView->getShuffleActive());
             m_pTrackView->show();
         }
 
@@ -410,6 +430,10 @@ void MainWindow::OnMainMenuAction(QAction* action)
         m_pSettingsDlg->readSettings();
         m_pSettingsDlg->showMaximized();
     }
+    else if(action->text()==tr("FM Transmitter"))
+    { //display the FM transmitter control panel dialog (plugin)
+        OnDisplayFMTransmitterDialog();
+    }
     else if(action->text()==tr("About"))
     { //display the app About dialog
         m_AboutDlg.exec();
