@@ -12,18 +12,19 @@
 
 
 /**
- *  Spotify session class. Handles the session by logging in. Emits signals for whatever
+ *  Spotify session class. Handles the session functions. Emits signals for whatever
  *  callback the Spotify library uses.
  */
 Spotify::Session* Spotify::Session::callingSession;
 
+
 Spotify::Session::Session( QObject* parent ):
     QObject( parent ),
     m_currentTrack( 0 ),
     m_playlistcontainer( 0 ),
     m_valid( false ),
     m_currentUser( 0 ),
-    m_trackPos( 0 )
+    m_playbackPos( 0 )
 {
     // Meta types:
     qRegisterMetaType<Spotify::Error>("Spotify::Error");
@@ -57,6 +58,8 @@ Spotify::Session::Session( QObject* parent ):
     connect(this, SIGNAL(onSearchComplete(sp_search*)),
                     this, SLOT(OnSearchComplete(sp_search*)),Qt::QueuedConnection);
 
+    connect(this, SIGNAL(onAlbumBrowserReady(sp_albumbrowse*,qint32*)),
+                    this, SLOT(OnAlbumBrowserReady(sp_albumbrowse*,qint32*)),Qt::QueuedConnection);
     m_pAudioThread = NULL;
 
     m_preBufActive = true;
@@ -64,7 +67,7 @@ Spotify::Session::Session( QObject* parent ):
     m_bPlaylistsLoading = false;
 }
 
-void Spotify::Session::init()
+void Spotify::Session::init(const sp_bitrate& bitrate)
 {
     //Init the current Spotify session.
     //Note: this method NEEDS to be initially called to set up the session, start audio playback thread etc.
@@ -81,7 +84,11 @@ void Spotify::Session::init()
     // Initiate session:
     m_error = sp_session_init( &m_config, &m_session );
     if( m_error.isError() )
+    {
         qDebug() << "FATAL ERROR: Could not initiate session";
+    }
+
+   sp_session_preferred_bitrate(m_session, bitrate); //set the preferred streaming bitrate
 
     m_ProcessEvents = false; //event processing not initiated
     m_DisableNotifyThread = false;
@@ -151,26 +158,14 @@ bool Spotify::Session::getPreBufferingActive()
 
 void Spotify::Session::setPlaybackPos(qint64 pos)
 {
-    m_trackPos = pos;
+    m_playbackPos = pos;
 }
 
 void Spotify::Session::setBufferingPos(qint64 pos)
 {
-    m_bufferPos = pos;
+    m_bufferedAmount = pos;
 }
 
-qint64 Spotify::Session::getBufferingPos()
-{
-    //returns the current track buffering position
-    if(!m_bufferPosList.isEmpty())
-    {
-        return m_bufferPosList.first();
-    }
-    else
-    {
-        return m_bufferPos;
-    }
-}
 
 void Spotify::Session::setPlaying(bool playing)
 {
@@ -273,9 +268,9 @@ void Spotify::Session::OnPlaybackFinished()
 void Spotify::Session::OnPlaybackPositionUpdated(qint64 duration)
 {
 
-    m_trackPos += duration;
+    m_playbackPos += duration;
 
-    emit playbackPositionUpdated(m_trackPos);
+    emit playbackPositionUpdated(m_playbackPos);
 }
 
 void Spotify::Session::resetPlaybackMembers()
@@ -342,7 +337,6 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
             m_pAudioThread->setNewTrackStarted(false);
             m_pAudioThread->setNextTrackStartPos(-1);
             m_pAudioThread->getMutex()->unlock();
-            m_bufferPosList.clear();
         }
     }
 
@@ -356,8 +350,8 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
     {
         if(!preBuffer) //track pre-buffer not active
         {
-            m_trackPos = 0;
-            m_bufferPos = 0;
+            m_playbackPos = 0;
+            m_bufferedAmount = 0;
         }
         m_playing = true;
     }
@@ -374,7 +368,7 @@ Spotify::Error Spotify::Session::seek( int pos )
 {
     Spotify::Error error = sp_session_player_seek( m_session, pos );
     if( !error.isError() )
-        m_trackPos = pos;
+        m_playbackPos = pos;
     return error;
  }
 
@@ -449,7 +443,7 @@ Spotify::Error Spotify::Session::search( const QString& query, const int trackOf
     return error;
 }
 
-Spotify::Error Spotify::Session::getAlbumCover( Spotify::Album* album )
+Spotify::Error Spotify::Session::getAlbumCover( Spotify::Album* album, qint32* pCallerID)
 {
     Spotify::Error error;
     if( album && album->getCover() != 0 )
@@ -457,7 +451,7 @@ Spotify::Error Spotify::Session::getAlbumCover( Spotify::Album* album )
         sp_image* image = sp_image_create( m_session, album->getCover() );
         error = sp_image_error( image );
         Spotify::ImageContainer* ic = dynamic_cast< Spotify::ImageContainer* >( album );
-        sp_image_add_load_callback( image, &Spotify::Session::imageLoadedCallback, ic );
+        sp_image_add_load_callback( image, &Spotify::Session::imageLoadedCallback, (void*)pCallerID );
     }
     else
         error = SP_ERROR_RESOURCE_NOT_LOADED;
@@ -570,7 +564,7 @@ Spotify::Error Spotify::Session::getPlaylistTracks(Spotify::Playlist* pPlaylist)
           {
             //attempt to load coverart for the track album
             qDebug()<<"Requesting coverart..";
-            getAlbumCover(track->getAlbum());
+            //getAlbumCover(track->getAlbum());
           }
 	}
       }
@@ -687,24 +681,27 @@ void Spotify::Session::artistCatalogueDeletedSlot( QObject* object )
         qDebug() << "WARNING: Trying to delete non-existing artist browser";
 }
 
-Spotify::Error Spotify::Session::browse( Spotify::Album* album )
+bool Spotify::Session::browse( Spotify::Album* album, qint32* pCallerID )
 {
-    Spotify::Error error;
     if( album )
     {
         sp_album* spalbum = album->getAlbum();
         sp_albumbrowse* ab = sp_albumbrowse_create( m_session, spalbum,
                                                     &Spotify::Session::albumBrowseCompleteCallback,
-                                                    NULL);
-        error = sp_albumbrowse_error( ab );
+                                                    (void*)pCallerID);
+        if(!ab)
+            return false; //failed to create browse album request
+
+        //error = sp_albumbrowse_error( ab );
         Spotify::AlbumBrowser* alb = new Spotify::AlbumBrowser( ab, this );
         m_albumBrowseHash.insert( ab, alb);
         connect( alb, SIGNAL(destroyed(QObject*)),
                  this, SLOT(albumBrowserDeletedSlot(QObject*)));
     }
     else
-        error = SP_ERROR_RESOURCE_NOT_LOADED;
-    return error;
+        return false;
+
+    return true;
 }
 
 void Spotify::Session::albumBrowserDeletedSlot( QObject* object )
@@ -756,14 +753,24 @@ void Spotify::Session::albumBrowseCompleteCallback( sp_albumbrowse* ab, void* us
 
 void Spotify::Session::albumBrowseCompleteWrapper( sp_albumbrowse* ab, void* userdata )
 {
-    Q_UNUSED( userdata );
+    qint32* pCallerID = (qint32*)userdata;
+    emit onAlbumBrowserReady(ab, pCallerID);
+}
+
+void Spotify::Session::OnAlbumBrowserReady(sp_albumbrowse *ab, qint32* pCallerID)
+{
+    qint32 callerID = -1;
+
+    if(pCallerID)
+    {
+        callerID = *(qint32*)pCallerID;
+    }
     if( ab && m_albumBrowseHash.contains( ab ) )
     {
-        qDebug() << "An album was just browser!";
+        qDebug() << "Album browse requeste completed!";
         Spotify::AlbumBrowser* alb = m_albumBrowseHash.value( ab );
-        emit albumBrowserReady( alb );
+        emit albumBrowserReady( alb, callerID);
     }
-
 }
 
 void SP_CALLCONV Spotify::Session::playlistAddedCallback( sp_playlistcontainer* pc, sp_playlist* playlist,
@@ -1084,9 +1091,9 @@ void Spotify::Session::imageLoadedWrapper( sp_image* image, void* userdata )
 void Spotify::Session::OnImageLoaded(sp_image* image, void* userdata)
 {
 
-   if( !image || !sp_image_is_loaded( image ) )
-   {
-       qDebug() << "Image not loaded..";
+    if( !image || !sp_image_is_loaded( image ) )
+    {
+        qDebug() << "Image not loaded..";
         return;
     }
 
@@ -1098,20 +1105,20 @@ void Spotify::Session::OnImageLoaded(sp_image* image, void* userdata)
     // Get image format:
     switch (spformat)
     {
-        case SP_IMAGE_FORMAT_JPEG:
-             format = "JPEG";
-             break;
-        case SP_IMAGE_FORMAT_UNKNOWN:
-            format = "UNKNOWN";
-            break;
-        default:
-            format = "UNKNOWN";
-            break;
+    case SP_IMAGE_FORMAT_JPEG:
+        format = "JPEG";
+        break;
+    case SP_IMAGE_FORMAT_UNKNOWN:
+        format = "UNKNOWN";
+        break;
+    default:
+        format = "UNKNOWN";
+        break;
     }
 
     if( format == "UNKNOWN" )
     {
-       qDebug()<< "Unknown format!";
+        qDebug()<< "Unknown format!";
         return;
     }
 
@@ -1120,29 +1127,25 @@ void Spotify::Session::OnImageLoaded(sp_image* image, void* userdata)
 
     // Create QImage instance, for wrapping image data
 
-   QImage img;
+    QImage img;
 
-    if(!img.loadFromData(data, (int)len, format.toAscii())) 
-     {
-       qDebug() << "Loading of image data failed!";
-      }
+    if(!img.loadFromData(data, (int)len, format.toAscii()))
+    {
+        qDebug() << "Loading of image data failed!";
+    }
 
 
     // Release sp_image:
     sp_image_release(image);
 
-    /*if(userdata)
-    {
-        qDebug() << "Assigning image container";
-        Spotify::ImageContainer* ic = (Spotify::ImageContainer*)userdata;
-        if( ic )
-        {
-            ic->setImage(img); 
-        }
-    }
-    */
+    qint32 callerID = -1;
 
-    emit coverArtReady(img);
+    if(userdata)
+    {
+      callerID = *(qint32*)userdata;
+    }
+
+    emit coverArtReady(img, callerID);
 }
 
 void Spotify::Session::searchCompleteCallback( sp_search* search, void* userdata )
@@ -1183,7 +1186,6 @@ void Spotify::Session::loggedInWrapper( Spotify::Error error )
 
     // Let application know we're logged in (or not):
     m_valid = !error.isError();
-    qDebug() << "Got here too..";
    m_currentUser = new Spotify::User( sp_session_user( m_session ) );
    qDebug() <<"User " + m_currentUser->getDisplayName() + " logged in..";
    emit loggedIn( error );
@@ -1227,13 +1229,13 @@ void Spotify::Session::connectionErrorWrapper( Spotify::Error error )
 void Spotify::Session::messageToUserCallback( sp_session* session, const char* message )
 {
     Q_UNUSED( session );
-    Spotify::Session::callingSession->messageToUserWrapper( QString( message ) );
+   // Spotify::Session::callingSession->messageToUserWrapper( QString( message ) );
 }
 
 void Spotify::Session::messageToUserWrapper( QString message )
 {
-    qDebug() << "MESSAGE (user): " << message.trimmed();
-    emit messageToUser( message.trimmed() );
+    //qDebug() << "MESSAGE (user): " << message.trimmed();
+    //emit messageToUser( message.trimmed() );
 }
 
 void Spotify::Session::notifyMainThreadCallback( sp_session* session )
@@ -1291,13 +1293,13 @@ void Spotify::Session::OnMusicDelivery(int num_frames, const sp_audioformat* for
             qDebug() << "Pre-buffering active; next track started...";
             m_pAudioThread->setNewTrackStarted(false); //reset track started state
             m_pAudioThread->setNextTrackStartPos();
-            m_bufferPosList.append(m_bufferPos);
-            m_bufferPos = 0; //reset buffering postion
+            m_bufferedAmount = 0; //reset buffering amount (newly loaded track)
         }
     }
 
-    m_bufferPos += (num_frames * 1000.0)/format->sample_rate;
-    emit trackPositionUpdated( m_bufferPos );
+    m_bufferedAmount += (num_frames * 1000.0)/format->sample_rate; //increase the currently buffered amount
+
+    emit trackBufferingAmountUpdated( m_bufferedAmount );
 
     //split audio data buffer into max 1 sec chunks
 
@@ -1309,6 +1311,7 @@ void Spotify::Session::OnMusicDelivery(int num_frames, const sp_audioformat* for
 
     m_pAudioThread->incFIFOSize(buffer.size());
 
+
     if(m_pAudioThread->getFIFOSize()>=(m_pAudioThread->getMinBufferSize()+5)*format->sample_rate*sizeof(int16_t)*format->channels) //>=35 secs of data in buffer
     {
         m_pAudioThread->setBufferingPaused(true);
@@ -1359,18 +1362,15 @@ void Spotify::Session::endOfTrackCallback( sp_session* session )
 void Spotify::Session::endOfTrackWrapper()
 {
     qDebug()<<"End of track callback..";
-    m_pAudioThread->getMutex()->lock();
     if(!m_preBufActive)
     {
+        m_pAudioThread->getMutex()->lock();
         m_pAudioThread->setEndOfTrack(true);
+        m_pAudioThread->getMutex()->unlock();
     }
     else
     {
-        if(!m_bufferPosList.isEmpty())
-            m_bufferPosList.removeFirst();
         emit bufferNextTrack(); //pre-buffering active; signal that buffering of following track can be initiated
     }
-
-    m_pAudioThread->getMutex()->unlock();
 }
 
