@@ -25,6 +25,7 @@
 
 namespace Spotify
 {
+
     class Session: public QObject
     {
         Q_OBJECT
@@ -33,7 +34,7 @@ namespace Spotify
             Session( QObject* parent = 0 );
             virtual ~Session();
 
-            void init(); //init the current Spotify session
+            void init(const sp_bitrate& bitrate = SP_BITRATE_160k); //init a new Spotify session (optional parameter the preferred streaming bitrate)
 
             void initAudio(); //init the audio / playback thread
 
@@ -55,8 +56,6 @@ namespace Spotify
 
             void setBufferingPos(qint64 pos);
 
-            qint64 getBufferingPos(); //returns the current track buffering position
-
             void setPlaying(bool playing);
 
             void resetPlaybackMembers();  //reset the playback related instance members (and removes any pending events from the playback thread)
@@ -159,12 +158,12 @@ namespace Spotify
                                    const int artistCount, void* userData );
 
             /**
-             * Get cover art for a given track.
-             * @param track Track to get cover art for.
+             * Get cover art for a given album.
+             * @param album Album to get cover art for.
+             * @param pCallerID Pointer to caller identifier (used to identify ID of receiver of result).
              * @return Error
              */
-            Spotify::Error getAlbumCover( Spotify::Album* album );
-            //Spotify::Error getAlbumCover( Spotify::Track* track );
+            Spotify::Error getAlbumCover( Spotify::Album* album, qint32* pCallerID);
 
             /**
              * Get the collection of playlists.
@@ -216,9 +215,10 @@ namespace Spotify
             /**
              * Browse an album
              * @param album Album to be browsed. The album metadata does not have to be loaded.
-             * @return Error code.
+             * @param pCallerID Pointer to identifier of of the caller (requester)
+             * @return TRUE on success, otherwise FALSE.
              */
-            Spotify::Error browse( Spotify::Album* album );
+            bool browse( Spotify::Album* album, qint32* pCallerID);
 
 
             //libopenspotify callbacks
@@ -312,6 +312,7 @@ namespace Spotify
             void OnImageLoaded(sp_image* image, void* userdata);
             void OnMusicDelivery(int num_frames, const sp_audioformat* format, QByteArray buffer);
             void OnSearchComplete(sp_search* search);
+            void OnAlbumBrowserReady(sp_albumbrowse* ab, qint32*);
             void OnResumeBuffering();
             void OnPlaybackPositionUpdated(qint64 duration);
             void OnPlaybackFinished();
@@ -352,7 +353,7 @@ namespace Spotify
 
             void artistCatalogueReady( Spotify::ArtistCatalogue* catalogue );
 
-            void albumBrowserReady( Spotify::AlbumBrowser* albumBrowser );
+            void albumBrowserReady( Spotify::AlbumBrowser*, qint32 );
             /**
              * Signal emitted when a message to the user has been received.
              */
@@ -361,8 +362,9 @@ namespace Spotify
             /**
              * Signal emitted when cover art has been received.
              * @param image QImage representation of the cover art.
+             * @param callerID ID of intended receiver of result.
              */
-            void coverArtReady( QImage image );
+            void coverArtReady( QImage image, qint32 callerID );
 
             /**
              * Signal emitted when the root playlist container has been loaded (internal usage)
@@ -442,14 +444,14 @@ namespace Spotify
             void playing(bool);
 
             /**
-             * Signal emitted when the current buffering amount position has been updated.
-             * @param pos The position in the buffer in milliseconds.
+             * Signal emitted when the current buffering amount has been updated.
+             * @param bufferedAmout The amount buffered of the currently loaded track (in milliseconds).
              */
-            void trackPositionUpdated( qint64 pos );
+            void trackBufferingAmountUpdated( qint32 bufferedAmount );
 
             /**
-             * Signal emitted when the current position in the track has been updated.
-             * @param pos The position in the track in milliseconds.
+             * Signal emitted when the current playback position in the track has been updated.
+             * @param pos The current playback position in the track (in milliseconds).
              */
             void playbackPositionUpdated( qint64 pos );
 
@@ -470,6 +472,8 @@ namespace Spotify
 
             void onSearchComplete(sp_search*);
 
+            void onAlbumBrowserReady(sp_albumbrowse*, qint32*);
+
             void bufferNextTrack();
 
 
@@ -487,11 +491,10 @@ namespace Spotify
             QString m_password;
             Spotify::User* m_currentUser;
 
-            qint64 m_bufferPos; //current buffering position
-            qint64 m_trackPos; //current track playback position
-            bool m_playing;
+            qint32 m_bufferedAmount; //current buffered amount (of the currently loaded track)
+            qint64 m_playbackPos; //current playback position for the currently loaded track
 
-            QList<qint64> m_bufferPosList; //used for track pre-buffering; list of track buffering states
+            bool m_playing;
 
             // For mapping sp_playlist pointers to Spotify::Playlist pointers
             QHash< sp_playlist*, Spotify::Playlist* > m_playlistHash;
